@@ -10,9 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import atexit
 import os
-import tempfile
 
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp import constants
@@ -20,31 +18,60 @@ from ovsdbapp.tests import base
 from ovsdbapp import venv
 
 
-class FunctionalTestCase(base.TestCase):
-    _connections = None
-    ovsvenv = venv.OvsOvnVenvFixture(tempfile.mkdtemp(),
-                                     ovsdir=os.getenv('OVS_SRCDIR'),
-                                     ovndir=os.getenv('OVN_SRCDIR'),
-                                     remove=not bool(os.getenv('KEEP_VENV')))
-    atexit.register(ovsvenv.cleanUp)
-    ovsvenv.setUp()
-    schema_map = {'Open_vSwitch': ovsvenv.ovs_connection,
-                  'OVN_Northbound': ovsvenv.ovnnb_connection,
-                  'OVN_Southbound': ovsvenv.ovnsb_connection,
-                  'OVN_IC_Northbound': ovsvenv.ovn_icnb_connection,
-                  }
-    ovsvenvlog = None
-    if os.getenv('KEEP_VENV') and os.getenv('VIRTUAL_ENV'):
-        ovsvenvlog = open(os.path.join(os.getenv('VIRTUAL_ENV'),
-                                       'ovsvenv.%s' % os.getpid()), 'a+')
-        atexit.register(ovsvenvlog.close)
-        ovsvenvlog.write("%s\n" % ovsvenv.venv)
+class VenvPerClassFunctionalTestCase(base.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.virtualenv = venv.VenvFixture(
+            remove=not bool(os.getenv("KEEP_VENV")))
+        cls.virtualenv.setUp()
+        cls.ovsvenvlog = None
+        if os.getenv('KEEP_VENV') and os.getenv('VIRTUAL_ENV'):
+            cls.ovsvenvlog = open(
+                os.path.join(os.getenv('VIRTUAL_ENV'),
+                             'ovsvenv.%s' % os.getpid()), 'a+')
+            cls.ovsvenvlog.write("%s\n" % cls.ovsvenv.venv)
+
+    @classmethod
+    def tearDownClass(cls):
+        if cls.ovsvenvlog:
+            cls.ovsvenvlog.close()
+        cls.virtualenv.cleanUp()
+        super().tearDownClass()
 
     @classmethod
     def venv_log(cls, val):
         if cls.ovsvenvlog:
             cls.ovsvenvlog.write("%s\n" % val)
 
+    def setUp(self):
+        super().setUp()
+        self.venv_log(self.id())
+
+
+class FunctionalTestCase(VenvPerClassFunctionalTestCase):
+    _connections = None
+    fixture_class = venv.OvsOvnVenvFixture
+
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.ovsvenv = cls.fixture_class(
+            cls.virtualenv,
+            ovsdir=os.getenv('OVS_SRCDIR'),
+            ovndir=os.getenv('OVN_SRCDIR'))
+        cls.ovsvenv.setUp()
+        cls.schema_map = {'Open_vSwitch': cls.ovsvenv.ovs_connection,
+                          'OVN_Northbound': cls.ovsvenv.ovnnb_connection,
+                          'OVN_Southbound': cls.ovsvenv.ovnsb_connection,
+                          }
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.ovsvenv.cleanUp()
+        super().tearDownClass()
+
     @property
     def connection(self):
         if len(self.schemas) == 1:
@@ -65,6 +92,5 @@ class FunctionalTestCase(base.TestCase):
         return connection.Connection(idl, constants.DEFAULT_TIMEOUT)
 
     def setUp(self):
-        super(FunctionalTestCase, self).setUp()
-        self.venv_log(self.id())
+        super().setUp()
         self.set_connection()
